var Fs = require('fs');
var Core = require('core');
var Request = require('core').Net.Request;
var Response = require('core').Net.Response;
var Router = require('./router/Router.js').Router;
var BinaryData = require('core').Data.BinaryData;
var Mime = require('core').Data.Mime;
var JsonRpcMethod = Core.JsonRpc.JsonRpcMethod;

var QueryMethod = function () {
    this.Extends = Core.JsonRpc.JsonRpcMethod;
    this.errorsCount = 0;
    this.execute = function (params) {
        if(params.hasOwnProperty('request')){
            var request = new Request(params.request);
            this.server.router.addEventListener(Router.Event.COMPLETE, this._onRoutingComplete, this);
            this.server.router.addEventListener(Router.Event.ERROR, this._onRoutingError, this);
            
            this.server.staticRouter.addEventListener(Router.Event.COMPLETE, this._onStaticRoutingComplete, this);
            this.server.staticRouter.addEventListener(Router.Event.ERROR, this._onRoutingError, this);
            
            this.server.resourceRouter.addEventListener(Router.Event.COMPLETE, this._onResourceRoutingComplete, this);
            this.server.resourceRouter.addEventListener(Router.Event.ERROR, this._onRoutingError, this);
            
            this.errorsCount = 0;
            
            this.server.router.run(request);
            this.server.staticRouter.run(request); 
            this.server.resourceRouter.run(request); 
        }
        else{
            this._onRoutingError();
        }
    };
    this._onRoutingComplete = function(e) {
        console.log("    -- Routing: Route Found");
        this._removeRouterListeners();
        if(this.server.router.actions.hasOwnProperty(e.data.name)){
            var action = this.server.router.actions[e.data.name];
            var response = new Response();
            response.setStatusCode(200);
            response.setHeader("cache-control", "no-cache", true);
            response.setHeader("content-type", "text/html; charset=utf-8", true);
            response.setHeader("x-onet-use", "include-engine", true);
            response.setBody(new BinaryData(action, BinaryData.Encoding.TEXT, BinaryData.CharacterEncoding.UTF8));
            this.dispatchEvent(new Core.Event.Event(Core.JsonRpc.JsonRpcMethod.Event.OK, response.toJson()));   
        }
        else{
            console.log("Problem: Route found but missing action.")
            this._onRoutingError();
        }
    };
    this._onRoutingError = function(e) {
        console.log("    -- Routing: Route Not found", this.errorsCount);
        if(this.errorsCount > 1){
            console.log("    -- Routing: Route Not found");
            this._removeRouterListeners();
             this._sendNotFound();
        }
        else{
            this.errorsCount++;
        }   
    };
    this._removeRouterListeners = function(){
        this.server.router.removeEventListener(Router.Event.COMPLETE, this._onRoutingComplete, this);
        this.server.router.removeEventListener(Router.Event.ERROR, this._onRoutingError, this);
        this.server.staticRouter.removeEventListener(Router.Event.COMPLETE, this._onStaticRoutingComplete, this);
        this.server.staticRouter.removeEventListener(Router.Event.ERROR, this._onRoutingError, this);
        this.server.resourceRouter.removeEventListener(Router.Event.COMPLETE, this._onResourceRoutingComplete, this);
        this.server.resourceRouter.removeEventListener(Router.Event.ERROR, this._onRoutingError, this);
    };
    
    this._onStaticRoutingComplete = function(e){
        console.log("    -- Routing: Static Route Found");
        this._removeRouterListeners();
        try {
            console.log("reading file:", e.data.name);
            var resource = Fs.readFileSync(e.data.name);
            var response = new Response();   
            response.setStatusCode(200);
            var contentType = Mime.getType(e.data.name); 
            response.setHeaders({
                'Content-Type': contentType,
                'Cache-Control': 'no-cache'
            }, true);
            response.setBody(new BinaryData( resource, BinaryData.Encoding.BINARY ));
            this._response.setStatusCode(200);
            this._response.setHeaders({
                'Access-Control-Allow-Origin': '*'
            });
            this._response.setBody(response.toBinaryData());
            this.dispatchEvent(new Core.Event.Event(Core.JsonRpc.JsonRpcMethod.Event.OK));
        }
        catch(err){console.log(err.message, err.stack)}
        //TODO - szitowe odwolanie do prywatnej z klasy nadrzednej
    };
    this._onResourceRoutingComplete = function(e){
        console.log("    -- Routing: Resource Route Found");
        this._removeRouterListeners();
        //TODO call server method
        //console.log("_onResourceRoutingComplete");
        var ResourceMethod = this.server.getMethod('resource');
        var method = new ResourceMethod(this._request, this._response, this.server);
        console.log('after instance of method')
        try {
            method.addEventListener(JsonRpcMethod.Event.WRONG_PARAMS, this._resourceMethodError, this);
            method.addEventListener(JsonRpcMethod.Event.ERROR, this._resourceMethodError, this)
            method.addEventListener(JsonRpcMethod.Event.OK, this._resourceMethodSuccess, this);   
        }
        catch(err){
            console.log('err', err)
        }
        console.log('after listeners')
        method.execute({resource: e.data.name, params: e.data.params});
    };
    
    this._resourceMethodSuccess = function(e){
        console.log('ResourceMethod - OK');
        try {
            //this._response.setBody(e.data);
            this.dispatchEvent(new Core.Event.Event(Core.JsonRpc.JsonRpcMethod.Event.OK));
        }
        catch(err){console.log("_resourceMethodSuccess", err.message, err.stack)}
        
    };
    
    this._resourceMethodError = function(e){
        console.log('ResourceMethod - ERROR');
        this._sendNotFound();
    };
    
    this._sendNotFound = function(){
        var response = new Response();
            response.setStatusCode(404);
            response.setHeader("cache-control", "no-cache", true);
            response.setHeader("content-type", "text/html; charset=utf-8", true);
            response.setBody(new BinaryData("404 - Takiej strony niema.", BinaryData.Encoding.TEXT, BinaryData.CharacterEncoding.UTF8));
            this.dispatchEvent(new Core.Event.Event(Core.JsonRpc.JsonRpcMethod.Event.OK, response.toJson()));   
    };
};

QueryMethod = new Core.Class(new QueryMethod());
exports.QueryMethod = QueryMethod;