var Core = require('core');
var RenderingController = require("./RenderingController.js").RenderingController;
var PluginManager = require('./plugins/PluginManager.js').PluginManager;
var Response = require('core').Net.Response;
var BinaryData = require('core').Data.BinaryData;


var ResourceMethod = function () {
    this.Extends = Core.JsonRpc.JsonRpcMethod;
    
    this.execute = function (params) {
        
        var config = this.server.getPluginsConfig();
        
        if(!params || !params.hasOwnProperty('resource')){
            console.log("     - ERROR: there is no param 'resource'");
            this.dispatchEvent(new Core.Event.ErrorEvent(Core.JsonRpc.JsonRpcMethod.Event.ERROR, null, -31999, "there is no param 'resource'"));
            return;
        }
        
        //console.log(params);
        if(!params || !config.hasOwnProperty(params.resource)){
            console.log("     - ERROR: there is no file: " + params.resource);
            this.dispatchEvent(new Core.Event.ErrorEvent(Core.JsonRpc.JsonRpcMethod.Event.ERROR, null, -31999, 'there is no file ' + params.resource));
            return;
        }
        
        var conf = config[params.resource];
        var that = this;
        
        var pluginManager = new PluginManager();
        
        pluginManager.addEventListener(PluginManager.Event.READY, function(e){
            console.log("     -- All Plugins are READY!");
            
            var response = new Response();   
            
            response.setStatusCode(200);
            response.setHeaders({
                'Content-Type': conf['class'].contentType,
                'Cache-Control': 'no-cache'
            }, true);
            
            response.setBody(new BinaryData( e.data, BinaryData.Encoding.BINARY ));
            that._response.setStatusCode(200);
            that._response.setHeaders({
                'Access-Control-Allow-Origin': '*'
            });
            that._response.setBody(response.toBinaryData());
            that.dispatchEvent(new Core.Event.Event(Core.JsonRpc.JsonRpcMethod.Event.OK));
        });
        
        pluginManager.addEventListener(PluginManager.Event.ERROR, function(e){
            console.log('     - Plugins: ERROR: ');
            that.dispatchEvent(new Core.Event.ErrorEvent(Core.JsonRpc.JsonRpcMethod.Event.ERROR, e.data.data, e.data.code, e.data.message));
        });
        
        for(var i=0,max=conf['class']['plugins'].length;i<max;i++){
            var ClassName = conf['class']['plugins'][i];
            var PluginPath = './plugins/' + ClassName + '.js'
            var PluginFile = require(PluginPath);
            var PluginClass = PluginFile[ClassName];
            var newObj = new PluginClass(conf);
            pluginManager.add(newObj);
        }
        
        pluginManager.start("", params);        
    };
    
};

ResourceMethod = new Core.Class(new ResourceMethod());
exports.ResourceMethod = ResourceMethod;