var Class = require('core').Class;
var Yaml = require('core').Yaml.YAML;
var cdf = require('dl').Cdf;
var CdfBlock = cdf.CdfBlock;
var CdfTemplate = cdf.CdfTemplate;
var CdfDatasource = cdf.CdfDatasource;
var CdfConfig = cdf.CdfConfig;


var CdfYaml = function(){};
CdfYaml = new Class( new CdfYaml() );

CdfYaml.parse = function(data){
    var yaml = Yaml.parse(data);
    var cdf = CdfYaml._convertToCdf(yaml);
    return cdf;
};

/*CdfYaml.stringify = function(obj){
    return Yaml.stringify(obj);
};*/

CdfYaml.parseFileSync = function(filepath){
    var yaml = Yaml.parseFileSync(filepath)[0];
    //console.log(filepath, require('util').inspect(yaml, true, 20));
    var cdf = CdfYaml._convertToCdf(yaml);
    return cdf;
};


CdfYaml._convertToCdf = function(config) {
    var name, cdf ="", block = null, template = null, cdfConfig = null;
    if(config){
        if(config.hasOwnProperty('blocks')){
            for(name in config.blocks){
                block = new CdfBlock();
                block.name = name;
                block.contentType = config.blocks[name].hasOwnProperty('contentType')? config.blocks[name].contentType : 'application/json';
                block.cache = config.blocks[name].hasOwnProperty('cache')? config.blocks[name].cache : 'none';
                block.onError = config.blocks[name].hasOwnProperty('onError')? config.blocks[name].onError : 'propagate';
                
                if(config.blocks[name].hasOwnProperty('template') && config.blocks[name].template != null){
                    if(block.contentType == 'text/html'){
                        block.setBody(config.blocks[name].template);
                    }
                    else{
                        template = CdfYaml._createCdfTemplate(config.blocks[name].template);
                        block.setBody(template.toString());   
                    }    
                }
                else{
                    //TODO error
                    console.log("Error - cdf:block doesn't have template defined.");
                }
                
                cdf += block.toString();
            }        
        }
        if(config.hasOwnProperty('config')){
            cdfConfig = new CdfConfig();
            if(config.config.hasOwnProperty('main')){
                cdfConfig.main = config.config.main;
            }
            cdf += cdfConfig.toString();
        }
    }
    return cdf;
};
    
CdfYaml._createCdfTemplate = function(data){
    var tpl = new CdfTemplate(data.name);
    tpl.role = data.role;
    tpl.kind = data.kind;
    if(data.hasOwnProperty('data') && data.data != null){
        if(data.data.hasOwnProperty('#body')){
            var children = data.data['#body'];
            for(var i = 0; i < children.length; i++){
                var child = CdfYaml._createCdfTemplate(children[i]);
                tpl.addChild(child);
            }
            delete data.data['#body'];
        }
        tpl.data = data.data;
    }
    if(data.hasOwnProperty('#data') && data['#data'] != null){
        tpl.data = new CdfDatasource(data['#data']);
    }
    return tpl;
};
CdfYaml.Exception = {};

exports.CdfYaml = CdfYaml;
