var Class           = require("core").Class;
var JsonRpcResponse = require("core").JsonRpc.JsonRpcResponse;

var Template = require('./Template.js').Template;
var jqtpl    = require('jqtpl');


/**
 * @class JqTemplate Implementation of Template class - using <a href="https://github.com/kof/node-jqtpl">jQuery Templates engine</a>.  
 * @extends Template
 * @requires Class 
 * @requires Template 
 * @requires jqtpl 
 * @requires JsonRpcResult
 */
var JqTemplate = function(){

    /** @ignore */
    this.Extends = Template;

    /**
     * Executing template rendering.
     * @param {Object} data Template data.
     * @throws {Template.Error.FILE_DOES_NOT_EXIST} if given template path is invalid.
     */
    this.render = function(data){
        //FIXME: template nie powinien uwzgledniac pakowania calosci w zwrotke RPC
        if(!jqtpl.template.hasOwnProperty(this._configuration.template)){
            throw Template.Exception.FILE_DOES_NOT_EXIST(this._configuration.template);
        }
        
        var dataObjects = {'data': data};
        for(var prop in this._api){
            dataObjects[prop] = this._api[prop];
        }
        
        var renderedData = jqtpl.tmpl(this._configuration.template, dataObjects, {cache: true});
        
        this.setRenderedData(renderedData);
    };
};

JqTemplate = new Class(new JqTemplate());
exports.JqTemplate = JqTemplate;